<?php
/*
Plugin Name: Fix API URL
Plugin URI: https://ederson.ferreira.tec.br
Description: Uses site_url() instead of home_url() to build the API URL. Useful for having WordPress as a Headless CMS.
Author: Ederson Peka
Version: 0.5
Author URI: https://profiles.wordpress.org/edersonpeka/
Text Domain: fix-api-url
*/

// Inspiration:
// https://wpti.ps/how-to-get-wordpress-rest-api-to-use-site-url-instead-of-home-url/

if ( !class_exists( 'fixapiurl' ) ) :

class fixapiurl {

    // change from HOME_URL (external) to SITE_URL (local)
    public static function home_to_site( $url ) {
        $url = str_replace( home_url(), site_url(), $url );
        return $url;
    }

    // change from SITE_URL (local) to HOME_URL (external)
    public static function site_to_home( $url ) {
        $url = str_replace( site_url(), home_url(), $url );
        return $url;
    }

    // change unpublished posts URL base to SITE_URL (local)
    public static function fix_unpublished_post_url( $url, $post_id ) {
        if ( get_post_status( $post_id ) != 'publish' ) {
            $url = call_user_func( array( __CLASS__, 'home_to_site' ), $url );
        }
        return $url;
    }

    public static function init() {
        // Internationalization
        load_plugin_textdomain( 'fix-api-url', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
        
        // Hooks
        add_filter( 'rest_url', array( __CLASS__, 'home_to_site' ) );
        add_filter( 'mod_rewrite_rules', array( __CLASS__, 'home_to_site' ) );
        add_filter( 'preview_post_link', array( __CLASS__, 'home_to_site' ) );
        foreach( [ 'post', 'page', 'attachment', 'post_type' ] as $type ) {
            add_filter( $type . '_link', array( __CLASS__, 'fix_unpublished_post_url' ), 10, 2 );
        }
        add_filter( 'admin_url', array( __CLASS__, 'home_to_site' ) );
        add_filter( 'includes_url', array( __CLASS__, 'home_to_site' ) );
        add_filter( 'vhp_purgeme_path', array( __CLASS__, 'home_to_site' ) );
        add_filter( 'vhp_purge_urls', function ( $a ) {
            $ret = array();
            foreach ( (array) $a as $u ) {
                $ret[] = call_user_func( array( __CLASS__, 'home_to_site' ), $u );
            }
            return $ret;
        } );

        // HOME_URL workaround to avoid API's 404 error

        // 1) set global variable so we can know we're parsing the request
        add_filter( 'do_parse_request', function ( $r ) {
            global $fix_api_url_parsing_request;
            $fix_api_url_parsing_request = true;
            return $r;
        } );

        // 2) if we're parsing the request, use site_url instead of home_url
        add_filter( 'home_url', function ( $u, $p ) {
            global $fix_api_url_parsing_request;
            if ( isset( $fix_api_url_parsing_request ) && $fix_api_url_parsing_request ) {
                $u = site_url( $p );
                $fix_api_url_parsing_request = false;
            }
            return $u;
        }, 0, 2 );
    }
}
add_action( 'init', array( 'fixapiurl', 'init' ), 0 );

endif;
